#include <gtk/gtk.h>
#include "moz_about.h"
#include "nsIIOService.h"
#include "nsIServiceManager.h"
#include "nsIStringStream.h"
#include "nsXPComFactory.h"
#include "nsNetUtil.h"

typedef enum {
    STREAM_ABOUT = 0,
    STREAM_MANUAL
} HtmlStream;

/**
 * this unpacks the compressed HTML data. This is in utils.c because zlib
 * header has some conflicts with NSPR types
 */
extern "C" char *uncompress_stream(int streamid);

static NS_DEFINE_CID(kIOServiceCID, NS_IOSERVICE_CID);

/* about:light */
class AboutLight : public nsIAboutModule 
{
public:
    NS_DECL_ISUPPORTS

    NS_DECL_NSIABOUTMODULE

    AboutLight() { NS_INIT_REFCNT(); }
    virtual ~AboutLight() {}

    static NS_METHOD
    Create(nsISupports *aOuter, REFNSIID aIID, void **aResult);

protected:
};

NS_IMPL_ISUPPORTS1(AboutLight, nsIAboutModule);

NS_IMETHODIMP
AboutLight::NewChannel(nsIURI *aURI, nsIChannel **result)
{
    nsresult rv;
    nsIChannel* channel;
    nsCOMPtr <nsIInputStream> in;
    char *dest;

    dest = uncompress_stream(STREAM_ABOUT);

    rv = NS_NewCStringInputStream(getter_AddRefs(in), nsDependentCString(dest));
    if (NS_FAILED(rv)) {
	g_free(dest);
	return rv;
    }

    rv = NS_NewInputStreamChannel(&channel, aURI, in,
	    NS_LITERAL_CSTRING("text/html"),
	    NS_LITERAL_CSTRING(""),
	    strlen(dest));

    if (NS_FAILED(rv)) {
	g_free(dest);
	return rv;
    }

    *result = channel;
    g_free(dest);
    return rv;
}

NS_METHOD
AboutLight::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
{
    AboutLight* about = new AboutLight();
    if (about == nsnull)
        return NS_ERROR_OUT_OF_MEMORY;
    NS_ADDREF(about);
    nsresult rv = about->QueryInterface(aIID, aResult);
    NS_RELEASE(about);
    return rv;
}

NS_DEF_FACTORY(AboutLight, AboutLight);

nsresult NS_NewAboutLightFactory(nsIFactory **aFactory)
{
    NS_ENSURE_ARG_POINTER(aFactory);
    *aFactory = nsnull;

    nsAboutLightFactory *result = new nsAboutLightFactory;
    if (result == nsnull)
	return NS_ERROR_OUT_OF_MEMORY;

    NS_ADDREF(result);
    *aFactory = result;
    return NS_OK;
}

/* about:manual */
class AboutManual : public nsIAboutModule 
{
public:
    NS_DECL_ISUPPORTS

    NS_DECL_NSIABOUTMODULE

    AboutManual() { NS_INIT_REFCNT(); }
    virtual ~AboutManual() {}

    static NS_METHOD
    Create(nsISupports *aOuter, REFNSIID aIID, void **aResult);

protected:
};

NS_IMPL_ISUPPORTS1(AboutManual, nsIAboutModule);

NS_IMETHODIMP
AboutManual::NewChannel(nsIURI *aURI, nsIChannel **result)
{
    nsresult rv;
    nsIChannel* channel;
    nsCOMPtr <nsIInputStream> in;
    char *dest;

    dest = uncompress_stream(STREAM_MANUAL);

    rv = NS_NewCStringInputStream(getter_AddRefs(in), nsCAutoString(dest));
    if (NS_FAILED(rv)) {
	g_free(dest);
	return rv;
    }

    rv = NS_NewInputStreamChannel(&channel, aURI, in, 
	    NS_LITERAL_CSTRING("text/html"),
	    NS_LITERAL_CSTRING(""),
	    strlen(dest));

    if (NS_FAILED(rv)) {
	g_free(dest);
	return rv;
    }

    *result = channel;
    g_free(dest);
    return rv;
}

NS_METHOD
AboutManual::Create(nsISupports *aOuter, REFNSIID aIID, void **aResult)
{
    AboutManual* about = new AboutManual();
    if (about == nsnull)
        return NS_ERROR_OUT_OF_MEMORY;
    NS_ADDREF(about);
    nsresult rv = about->QueryInterface(aIID, aResult);
    NS_RELEASE(about);
    return rv;
}

NS_DEF_FACTORY(AboutManual, AboutManual);

nsresult NS_NewAboutManualFactory(nsIFactory **aFactory)
{
    NS_ENSURE_ARG_POINTER(aFactory);
    *aFactory = nsnull;

    nsAboutManualFactory *result = new nsAboutManualFactory;
    if (result == nsnull)
	return NS_ERROR_OUT_OF_MEMORY;

    NS_ADDREF(result);
    *aFactory = result;
    return NS_OK;
}
