/* mozilla c++ bits
 * vim:ts=8:et:sw=2
 *
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <gtkmozembed.h>
#include <gtkmozembed_internal.h>
#include <wchar.h>

#include <nsEmbedAPI.h>

#include <nsString.h>

#include <nsIDocument.h>

#include <nsIDocShell.h>
#include <nsIDocShellTreeOwner.h>
#include <nsIDocShellTreeNode.h>

#include <nsIWebBrowser.h>
#include <nsCWebBrowser.h>
#include <nsCWebBrowserPersist.h>
#include <nsIWebBrowserFind.h>
#include <nsIWebBrowserFocus.h>
#include <nsIWebBrowserPersist.h>

#include <nsIContentViewer.h>
#include <nsIContentViewerFile.h>
#include <nsIPrintOptions.h>
#include <nsGfxCIID.h>          /* for NS_PRINTOPTIONS_CID */
#include <nsIMarkupDocumentViewer.h>

/* history stuff */
#include <nsISHistory.h>
#include <nsIGlobalHistory.h>
#include <nsIDocShellHistory.h>

#ifndef ENGLISH_IS_NO1
/* charset converter stuff */
#include <nsICharsetConverterManager.h>
#include <nsICharsetConverterManager2.h>
#endif

#include <nsIURI.h>
#include <nsIStreamTransfer.h>

#include <nsNetUtil.h>

/* clear memory / disk cache */
#include <nsICacheService.h>

#include <nsIDOMDocument.h>
#include <nsIDOMElement.h>
#include <nsIDOMNSDocument.h>
#include <nsIDOMMouseEvent.h>
#include <nsIDOMKeyEvent.h>
#include <nsIDOMEventTarget.h>
#include <nsIDOMNSEvent.h>                                                      
#include <nsIDOMCharacterData.h>
#include <nsIDOMHTMLElement.h>
#include <nsIDOMNSHTMLElement.h>
#include <nsIDOMHTMLInputElement.h>
#include <nsIDOMHTMLObjectElement.h>
#include <nsIDOMHTMLImageElement.h>
#include <nsIDOMHTMLAreaElement.h>
#include <nsIDOMHTMLAnchorElement.h>
#include <nsIDOMWindowInternal.h>
#include <nsIDOMNamedNodeMap.h>

#include <nsIInterfaceRequestor.h>

#include "light.h"
#include "moz_about.h"

/* local prototypes */
static char *moz_unicode_to_locale(const PRUnichar *uniStr);
static char *convert_ns_string_to_c_string(const nsString &ns_string);

static nsIDocShell *moz_get_docshell(GtkMozEmbed *embed);
static nsresult moz_get_dom_attr(nsIDOMNode * node, char *tag, char **attribute);

extern "C" void moz_init(char *mozdir)
{
  nsresult rv = NS_OK;
  static NS_DEFINE_CID(kAboutLightCID, ABOUT_LIGHT_MODULE_CID);
  static NS_DEFINE_CID(kAboutManualCID, ABOUT_MANUAL_MODULE_CID);

  /* about:light */
  nsCOMPtr < nsIFactory > aboutfactory;
  rv = NS_NewAboutLightFactory(getter_AddRefs(aboutfactory));
  if (NS_FAILED(rv))
    return;

  nsComponentManager::RegisterFactory(kAboutLightCID,
                                      "Light",
                                      ABOUT_LIGHT_CONTRACTID,
                                      aboutfactory, PR_TRUE);

  /* about:manual */
  nsCOMPtr < nsIFactory > manualfactory;
  rv = NS_NewAboutManualFactory(getter_AddRefs(manualfactory));
  if (NS_FAILED(rv))
    return;

  nsComponentManager::RegisterFactory(kAboutManualCID,
                                      "Manual",
                                      ABOUT_MANUAL_CONTRACTID,
                                      manualfactory, PR_TRUE);
}

extern "C" void moz_history_init(Browser *browser)
{
  nsCOMPtr < nsIDocShell > docshell;

  docshell = moz_get_docshell(GTK_MOZ_EMBED(browser->embed));

  if (!docshell)
    return;

  nsCOMPtr < nsIDocShellHistory > dsHistory = do_QueryInterface(docshell);
  if (dsHistory) {
    nsresult rv;
    nsCOMPtr < nsIGlobalHistory > history =
      do_GetService(NS_GLOBALHISTORY_CONTRACTID, &rv);
    if (history) {
      dsHistory->SetGlobalHistory(history);
    }
  }
}

#ifndef ENGLISH_IS_NO1
extern "C" int moz_get_charset(char ***charsets)
{
  nsresult rv;
  PRUint32 cscount;
  PRUint32 i;
  char **local = NULL;

  nsCOMPtr < nsIAtom > docCharsetAtom;
  nsCOMPtr < nsICharsetConverterManager2 > ccm2 =
      do_GetService(NS_CHARSETCONVERTERMANAGER_CONTRACTID, &rv);
  if (!NS_SUCCEEDED(rv))
    return 0;

  nsCOMPtr < nsISupportsArray > cs_list;
  rv = ccm2->GetDecoderList(getter_AddRefs(cs_list));
  if (!NS_SUCCEEDED(rv))
    return 0;

  rv = cs_list->Count(&cscount);
  local = g_new0(char *, cscount + 1);

  for (i = 0; i < cscount; i++) {
    nsCOMPtr < nsISupports > cssupports =
        (dont_AddRef) (cs_list->ElementAt(i));
    nsCOMPtr < nsIAtom > csatom(do_QueryInterface(cssupports));
    nsString charset_ns = nsnull, charset_title_ns = nsnull;

    /* charset name */
    rv = csatom->ToString(charset_ns);
    char *charset_str = convert_ns_string_to_c_string(charset_ns);
    local[i] = g_strdup(charset_str);
    g_free(charset_str);
  }
  *charsets = local;
  return cscount;
}
#endif

extern "C" int moz_find(Browser *browser, const char *exp, int matchcase,
                        int search_backwards, int *did_find)
{
  nsString searchString;
  wchar_t *dest = NULL;
  PRUnichar *unidest = NULL;
  int wc_size, i;
  bool found;

  /* get nsIBrowser */
  nsIWebBrowser *wb = nsnull;
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);
  nsCOMPtr < nsIWebBrowserFind > finder(do_GetInterface(wb));

  /* here, we try to convert locale-entered data into unicode */
  wc_size = mbstowcs(NULL, exp, 0);
  if (wc_size > 0) {
    dest = g_new0(wchar_t, wc_size + 1);
    unidest = g_new0(PRUnichar, wc_size + 1);
    mbstowcs(dest, exp, wc_size);
    for (i = 0; i < wc_size; i++) {
      unidest[i] = dest[i];
    }
  }

  /* do the unicode way, or if mbstowcs failed for some reason,
   * the normal way */
  if (unidest) {
    finder->SetSearchString(unidest);
  } else {
    searchString.AssignWithConversion(exp);
    finder->SetSearchString(searchString.get());
  }

  finder->SetFindBackwards(search_backwards);
  finder->SetMatchCase(matchcase);

  found = finder->FindNext(did_find);

  /* clear our temp stuff */
  if (unidest) {
    g_free(unidest);
    g_free(dest);
  }
  return found;
}

extern "C" void moz_print(Browser *browser, char *lpr, char *file)
{
  nsresult rv;
  static NS_DEFINE_CID(kPrintOptionsCID, NS_PRINTOPTIONS_CID);

  nsCOMPtr < nsIPrintSettings > options =
    do_GetService(kPrintOptionsCID, &rv);

  if (NS_FAILED(rv))
    return;

  options->SetMarginTop(0.5);
  options->SetMarginBottom(0.5);
  options->SetMarginLeft(0.5);
  options->SetMarginRight(0.5);

  nsCString printerString(lpr);
  nsCString fileString(file);

  options->SetToFileName(ToNewUnicode(fileString));
  options->SetPrintCommand(ToNewUnicode(printerString));
  /* print to file */
  if (file)
    options->SetPrintToFile(PR_TRUE);

  options->SetPrintSilent(PR_TRUE);

  /* actual print stuff */
  nsCOMPtr < nsIDocShell > DocShell;
  DocShell = moz_get_docshell(GTK_MOZ_EMBED(browser->embed));
  if (!DocShell)
    return;

  nsCOMPtr < nsIContentViewer > contentViewer;
  DocShell->GetContentViewer(getter_AddRefs(contentViewer));

  if (contentViewer) {
    nsCOMPtr < nsIContentViewerFile >
        contentViewerFile(do_QueryInterface(contentViewer));
    if (contentViewerFile) {
      rv = contentViewerFile->Print(PR_TRUE, nsnull, nsnull);
    }
  }
}

extern "C" void moz_set_size(Browser *browser, float size)
{
  nsresult rv;

  nsCOMPtr < nsIDocShell > DocShell;
  nsCOMPtr < nsIContentViewer > contentViewer;

  DocShell = moz_get_docshell(GTK_MOZ_EMBED(browser->embed));
  if (!DocShell)
    return;

  rv = DocShell->GetContentViewer(getter_AddRefs(contentViewer));
  if (!NS_SUCCEEDED(rv) || !contentViewer)
    return;

  nsCOMPtr < nsIMarkupDocumentViewer > mdv =
      do_QueryInterface(contentViewer, &rv);
  if (NS_FAILED(rv) || !mdv)
    return;

  mdv->SetTextZoom(size);
}

extern "C" gint moz_set_charset(Browser *browser, char *charset)
{
  nsresult result;

  nsCOMPtr < nsIDocShell > DocShell;
  DocShell = moz_get_docshell(GTK_MOZ_EMBED(browser->embed));

  if (!DocShell)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIContentViewer > contentViewer;
  result = DocShell->GetContentViewer(getter_AddRefs(contentViewer));
  if (!NS_SUCCEEDED(result) || !contentViewer)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIMarkupDocumentViewer > mdv =
      do_QueryInterface(contentViewer, &result);
  if (NS_FAILED(result) || !mdv)
    return NS_ERROR_FAILURE;

  nsAutoString charset_str;
  charset_str.AssignWithConversion(charset);
  result = mdv->SetForceCharacterSet(ToNewUnicode(charset_str));

  return result;
}

extern "C" void moz_save_link(Browser *browser, const char *url)
{
  nsresult rv;
  nsCOMPtr < nsIDOMWindow > DOMWindow;
  nsIWebBrowser *wb = nsnull;

  /* get nsIBrowser */
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);

  rv = wb->GetContentDOMWindow(getter_AddRefs(DOMWindow));

  if (NS_FAILED(rv) || !DOMWindow)
    return;

  nsCOMPtr < nsIDOMWindowInternal > DOMWInternal =
      do_QueryInterface(DOMWindow);
  if (!DOMWInternal)
    return;

  nsCOMPtr < nsIStreamTransfer > xfer =
      do_GetService(NS_ISTREAMTRANSFER_CONTRACTID);
  if (!xfer)
    return;

  xfer->SelectFileAndTransferLocationSpec(url, DOMWInternal, "", "",
                                          PR_TRUE, nsnull, nsnull);
}

extern "C" void moz_save_cached(Browser *browser, const char *uri,
                                const char *filename)
{
  nsresult rv;
  nsIWebBrowser *wb = nsnull;
  nsString s;

  /* get nsIBrowser */
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);

  s.AssignWithConversion(uri);

  nsCOMPtr < nsIURI > linkURI;

  rv = NS_NewURI(getter_AddRefs(linkURI), s);
  if (NS_FAILED(rv))
    return;

  nsCOMPtr<nsIWebBrowserPersist> persist = 
    do_CreateInstance(NS_WEBBROWSERPERSIST_CONTRACTID, &rv);
  if (NS_FAILED(rv) || !persist)
    return;

#if 0 /* TODO!!! */
  nsCOMPtr<nsILocalFile> file;
  NS_NewLocalFile(filename, PR_TRUE, getter_AddRefs(file));
  if (NS_FAILED(rv) || !file)
    return;

  persist->SaveURI(linkURI, nsnull, file);
#endif
}

extern "C" int moz_key_info(gpointer event, KeyEventInfo *info)
{
  PRBool mod_key;
  nsIDOMKeyEvent *keyEvent = (nsIDOMKeyEvent *) event;

  info->modifier = 0;

  keyEvent->GetCharCode((PRUint32 *) & info->key);

  if (info->key == 0) {
    keyEvent->GetKeyCode((PRUint32 *) & info->key);
    info->modifier |= KEY_CODE;
  }

  keyEvent->GetAltKey(&mod_key);
  if (mod_key)
    info->modifier |= ALT_KEY;

  keyEvent->GetShiftKey(&mod_key);
  if (mod_key)
    info->modifier |= SHIFT_KEY;

  keyEvent->GetMetaKey(&mod_key);
  if (mod_key)
    info->modifier |= META_KEY;

  keyEvent->GetCtrlKey(&mod_key);
  if (mod_key)
    info->modifier |= CTRL_KEY;

  return NS_OK;
}

nsresult GetLinkText(nsIDOMNode *rootnode, char **linktext)
{
	nsresult result;
	int depth = 1;
	nsAutoString ltext;

	nsCOMPtr<nsIDOMNode> node;
	result = rootnode->GetFirstChild (getter_AddRefs(node));
	if (NS_FAILED(result)) return NS_ERROR_FAILURE;

	while (node && depth > 0)
	{
		nsString name;
		node->GetNodeName(name);
		if (name.EqualsWithConversion("#text", PR_TRUE))
		{
			nsCOMPtr<nsIDOMCharacterData> cdata = do_QueryInterface(node);
			nsString ctext;
			cdata->GetData(ctext);
			nsString s;
			s.AssignWithConversion(" ");
			ltext += s;
			ltext += ctext;
		}

		PRBool childs;
		node->HasChildNodes(&childs);

		if (childs)
		{
			nsCOMPtr<nsIDOMNode> childNode;
			node->GetFirstChild (getter_AddRefs(childNode));
			node = childNode;

			depth ++;
		}
		else
		{
			nsCOMPtr<nsIDOMNode> siblingNode;
			result = node->GetNextSibling (getter_AddRefs(siblingNode));
			if (NS_FAILED(result))
			{
				nsCOMPtr<nsIDOMNode> parentNode;
				result = node->GetParentNode (getter_AddRefs(parentNode));
				if (!NS_FAILED(result))
				{
					parentNode->GetNextSibling (getter_AddRefs(siblingNode));
				}
			}
			
			node = siblingNode;

		}
	}

	gchar *tmp = ToNewCString (ltext);
	gchar *clean = g_strstrip (tmp);
	nsMemory::Free (tmp);

	if (strlen (clean) == 0)
	{
		g_free (clean);
		*linktext = NULL;
	}
	else
		*linktext = clean;

	return NS_OK;
}

extern "C" int moz_mouse_info(gpointer event, MouseEventInfo *info,
                              Browser *browser)
{
  nsresult rv;
  DOMTimeStamp ts;
  info->img = info->link = NULL;
  nsIDOMMouseEvent *aMouseEvent = (nsIDOMMouseEvent *) event;

  aMouseEvent->GetButton((PRUint16 *) & info->button);
  aMouseEvent->GetTimeStamp(&ts);
  info->timestamp = ts;

  /* be sure we are not clicking on the scrollbars */

  nsCOMPtr < nsIDOMEventTarget > OriginalTarget;
  nsCOMPtr<nsIDOMNSEvent> nsevent(do_QueryInterface(aMouseEvent));
  if (nsevent) {
    rv = nsevent->GetOriginalTarget(getter_AddRefs(OriginalTarget));
    if (NS_FAILED(rv)) {
      return NS_ERROR_FAILURE;
    }
  }

  nsCOMPtr < nsIDOMNode > OriginalNode = do_QueryInterface(OriginalTarget);
  if (!OriginalNode)
    return NS_ERROR_FAILURE;

  nsString nodename;
  OriginalNode->GetNodeName(nodename);

  if (nodename.EqualsWithConversion("xul:thumb") ||
      nodename.EqualsWithConversion("xul:slider"))
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIDOMEventTarget > EventTarget;
  rv = aMouseEvent->GetTarget(getter_AddRefs(EventTarget));
  if (NS_FAILED(rv) || !EventTarget)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIDOMNode > node = do_QueryInterface(EventTarget);
  if (!node)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIDOMDocument > domDoc;
  rv = node->GetOwnerDocument(getter_AddRefs(domDoc));
  if (!NS_SUCCEEDED(rv) || !domDoc)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIDocument > doc = do_QueryInterface(domDoc);
  if (!doc)
    return NS_ERROR_FAILURE;

  nsIURI *baseURI;
  rv = doc->GetBaseURL(baseURI);
  if (NS_FAILED(rv) || !baseURI)
    return NS_ERROR_FAILURE;

  nsCOMPtr<nsIDOMNSDocument> nsDoc = do_QueryInterface(domDoc, &rv);
  if (NS_FAILED(rv) || !nsDoc)
    return NS_ERROR_FAILURE;

  nsString mime;
  rv = nsDoc->GetContentType(mime);
  if (NS_FAILED(rv))
    return NS_ERROR_FAILURE;

  if (mime.EqualsWithConversion("text/xul"))
    return NS_OK;

  info->context = CONTEXT_NONE;
  nsCOMPtr < nsIDOMHTMLElement > element;
  do {
    PRUint16 type;
    node->GetNodeType(&type);

    element = do_QueryInterface(node);
    if (element) {
      nsAutoString tag;
      element->GetTagName(tag);

      if (tag.EqualsWithConversion("input", PR_TRUE)) {
        info->context |= CONTEXT_INPUT;
      } else if (tag.EqualsWithConversion("img", PR_TRUE)) {
        info->context |= CONTEXT_IMAGE;

        char *src;
        rv = moz_get_dom_attr(node, "src", &src);
        if (NS_FAILED(rv) || !src)
          return NS_ERROR_FAILURE;

        nsCAutoString csrc(src);
        nsCAutoString imginfo;

        rv = baseURI->Resolve(csrc, imginfo);
        info->img = g_strdup(imginfo.get());

        if (NS_FAILED(rv) || !info->img)
          return NS_ERROR_FAILURE;
        g_free(src);

      } else {
        info->context |= CONTEXT_OTHER;
      }

      nsCOMPtr < nsIDOMNamedNodeMap > attributes;
      node->GetAttributes(getter_AddRefs(attributes));
      if (attributes) {
        nsCOMPtr < nsIDOMNode > hrefNode;
        nsAutoString href;
        href.AssignWithConversion("href");
        attributes->GetNamedItem(href, getter_AddRefs(hrefNode));
        if (hrefNode) {
          info->context |= CONTEXT_LINK;
          nsAutoString linkhtml;
          nsCOMPtr < nsIDOMNSHTMLElement > nsElement;
          nsElement = do_QueryInterface(element);
          if (nsElement) {
            rv = nsElement->GetInnerHTML(linkhtml);
            if (NS_SUCCEEDED(rv)) {
              info->linktext =
                  moz_unicode_to_locale(linkhtml.get());
            }
          }
          char *href;
          rv = moz_get_dom_attr(node, "href", &href);
          if (NS_FAILED(rv) || !href)
            return NS_ERROR_FAILURE;

          nsCAutoString chref(href);
          nsCAutoString linkinfo;

          rv = baseURI->Resolve(chref, linkinfo);
          info->link = g_strdup(linkinfo.get());

          if (NS_FAILED(rv) || !info->link)
            return NS_ERROR_FAILURE;
          g_free(href);
          break;
        }
      }

    }
    nsCOMPtr < nsIDOMNode > parentNode;
    node->GetParentNode(getter_AddRefs(parentNode));

    if (!parentNode) {
      node = nsnull;
      info->context |= CONTEXT_DOCUMENT;
      break;
    }
    node = parentNode;
  }
  while (node);

  /* Get the modifier */
  PRBool mod_key;

  info->modifier = 0;

  aMouseEvent->GetAltKey(&mod_key);
  if (mod_key)
    info->modifier |= ALT_KEY;

  aMouseEvent->GetShiftKey(&mod_key);
  if (mod_key)
    info->modifier |= SHIFT_KEY;

  aMouseEvent->GetMetaKey(&mod_key);
  if (mod_key)
    info->modifier |= META_KEY;

  aMouseEvent->GetCtrlKey(&mod_key);
  if (mod_key)
    info->modifier |= CTRL_KEY;

  /* Find out if the page is framed */
  /* get web browser */
  nsIWebBrowser *wb = nsnull;
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);

  nsCOMPtr < nsIDocShellTreeItem > browserAsItem;
  browserAsItem = do_QueryInterface(wb);
  if (!browserAsItem)
    return NS_ERROR_FAILURE;

  /* get the owner for that item */
  nsCOMPtr < nsIDocShellTreeOwner > treeOwner;
  browserAsItem->GetTreeOwner(getter_AddRefs(treeOwner));
  if (!treeOwner)
    return NS_ERROR_FAILURE;

  /* get the primary content shell as an item */
  nsCOMPtr < nsIDocShellTreeItem > contentItem;
  treeOwner->GetPrimaryContentShell(getter_AddRefs(contentItem));
  if (!contentItem)
    return NS_ERROR_FAILURE;

  nsCOMPtr < nsIDocShellTreeNode > contentNode;
  contentNode = do_QueryInterface(contentItem);
  if (!contentNode)
    return NS_ERROR_FAILURE;

  PRInt32 childCount;
  contentNode->GetChildCount(&childCount);

  info->framed = (childCount > 0);

  return NS_OK;
}

/**
 * moz_get_main_document_url: Get URL of main document
 */
extern "C" char *moz_get_main_document_url(Browser *browser)
{
  nsresult rv;

  /* get web browser */
  nsIWebBrowser *wb = nsnull;
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);

  nsCOMPtr < nsIDocShell > docshell;
  docshell = moz_get_docshell(GTK_MOZ_EMBED(browser->embed));

  nsCOMPtr < nsIContentViewer > contentViewer;
  rv = docshell->GetContentViewer(getter_AddRefs(contentViewer));
  if (!NS_SUCCEEDED(rv) || !contentViewer)
    return NULL;

  nsCOMPtr < nsIDOMDocument > aDOMDocument;
  contentViewer->GetDOMDocument(getter_AddRefs(aDOMDocument));

  nsCOMPtr < nsIDocument > doc = do_QueryInterface(aDOMDocument);
  if (!doc)
    return NULL;

  nsCOMPtr <nsIURI> uri;
  doc->GetDocumentURL(getter_AddRefs(uri));

  nsCAutoString url;
  uri->GetSpec(url);
  return g_strdup(url.get());
}

/**
 * moz_get_document_url: Get URL of focused document (frame)
 */
extern "C" char *moz_get_document_url(Browser *browser)
{
  nsresult rv;

  /* get web browser */
  nsIWebBrowser *wb = nsnull;
  gtk_moz_embed_get_nsIWebBrowser(GTK_MOZ_EMBED(browser->embed), &wb);

  nsCOMPtr < nsIWebBrowserFocus > focus(do_GetInterface(wb));
  nsCOMPtr < nsIDOMWindow > DOMWindow;
  rv = focus->GetFocusedWindow(getter_AddRefs(DOMWindow));
  if (NS_FAILED(rv) || !DOMWindow)
    return NULL;

  nsCOMPtr < nsIDOMDocument > aDOMDocument;
  rv = DOMWindow->GetDocument(getter_AddRefs(aDOMDocument));
  if (NS_FAILED(rv) || !aDOMDocument)
    return NULL;

  nsCOMPtr < nsIDocument > doc = do_QueryInterface(aDOMDocument);
  if (!doc)
    return NULL;

  nsCOMPtr <nsIURI> uri;
  doc->GetDocumentURL(getter_AddRefs(uri));

  nsCAutoString url;
  uri->GetSpec(url);
  return g_strdup(url.get());
}

/**
 * moz_clear_cache: Clear the global memory/disk cache
 */
extern "C" void moz_clear_cache(unsigned long cache)
{
  nsCOMPtr < nsICacheService > CacheService =
      do_CreateInstance(NS_CACHESERVICE_CONTRACTID);
  CacheService->EvictEntries(cache);
}

static nsIDocShell *moz_get_docshell(GtkMozEmbed *embed)
{
  nsIDocShell *newshell;

  /* get nsIBrowser */
  nsIWebBrowser *wb = nsnull;
  gtk_moz_embed_get_nsIWebBrowser(embed, &wb);

  // do_GetInterface?
  nsCOMPtr < nsIDocShell > docshell(do_GetInterface(wb));
  newshell = docshell.get();
  NS_IF_ADDREF(newshell);

  return newshell;
}

static nsresult moz_get_dom_attr(nsIDOMNode *node, char *tag, char **attribute)
{
  nsresult result;

  nsCOMPtr < nsIDOMNamedNodeMap > attributes;
  result = node->GetAttributes(getter_AddRefs(attributes));
  if (!NS_SUCCEEDED(result) || !attributes)
    return NS_ERROR_FAILURE;

  nsAutoString attr;

  attr.AssignWithConversion(tag);

  nsCOMPtr < nsIDOMNode > attrNode;
  result = attributes->GetNamedItem(attr, getter_AddRefs(attrNode));
  if (!NS_SUCCEEDED(result) || !attrNode)
    return NS_ERROR_FAILURE;

  nsAutoString nodeValue;

  result = attrNode->GetNodeValue(nodeValue);
  if (!NS_SUCCEEDED(result))
    return NS_ERROR_FAILURE;

  char *cstr = ToNewCString(nodeValue);
  *attribute = g_strdup(cstr);

  nsMemory::Free(cstr);

  return NS_OK;
}

/* This nonsense is needed to get the allocators right */
static char *convert_ns_string_to_c_string(const nsString &ns_string)
{
  char *c_string;
  nsCString ns_c_string;

  ns_c_string.AssignWithConversion(ns_string);

  c_string = g_strdup (ns_c_string.get());

  return c_string;
}

/**
 * moz_unicode_to_locale: Encodes unicode string to something
 * valid for the current locale (which can then be used in GTK+ labels).
 * @uniStr: The unicode string to encode
 */
static char *moz_unicode_to_locale(const PRUnichar *uniStr)
{
  PRInt32 sSize;
  wchar_t *wide;
  char *output;
  int i, count;

  if (uniStr == NULL)
    return NULL;

  const nsString str(uniStr);
  sSize = str.Length();

  /* allocate a wide string big enough to hold the unicode string,
   * this is necessary since wchar_t is 32-bits with glibc */
  wide = g_new0(wchar_t, sSize + 1);
  for (i = 0; i < sSize + 1; i++) {
    wide[i] = uniStr[i];
  }

  /* use glibc function to determine the size of this string once
   * encoded to a locale specfic multibyte string */
  count = wcstombs(NULL, wide, 0);

  /* check for success */
  if (count == -1) {
    /* wcstombs failed - return asciified string */
    nsCString str;
    str.AssignWithConversion(uniStr);
    g_free(wide);
    return g_strdup(str.get());
  }

  /* allocate a string big enough and do the actual conversion */
  output = g_new0(gchar, count + 1);
  count = wcstombs(output, wide, count + 1);
  g_assert(count != -1);

  /* free wide version and return */
  g_free(wide);
  return output;
}

/**
 * moz_get_document_title: get the document title in a locale specific
 * NULL terminated C string, using the private UniCode interface.
 */
extern "C" char *moz_get_document_title(Browser *browser)
{
  PRUnichar *unicode_title;
  char *title;

  /* get the title in unicode */
  unicode_title = gtk_moz_embed_get_title_unichar
      (GTK_MOZ_EMBED(browser->embed));

  /* attempt conversion */
  title = moz_unicode_to_locale(unicode_title);

  /* free unicode version */
  g_free(unicode_title);

  /* return it */
  return title;
}
