#include <gtk/gtk.h>
#include <nsCOMPtr.h>
#include <nsNetUtil.h>
#include <nsIPrefService.h>

/* save prefs on exit */
extern "C" void moz_pref_save(void)
{
    nsCOMPtr <nsIPrefService> prefService = 
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    g_return_if_fail(prefService != nsnull);

    prefService->SavePrefFile(nsnull);
}

/* set prefs */
extern "C" void moz_pref_set_char(const char *name, const char *value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);

    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->SetCharPref(name, value);
}

extern "C" void moz_pref_set_bool(const char *name, const int value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);

    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->SetBoolPref(name, value ? PR_TRUE : PR_FALSE);
}

extern "C" void moz_pref_set_int(const char *name, const int value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->SetIntPref(name, value);
}

/* get prefs */
extern "C" void moz_pref_get_char(const char *name, char **value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->GetCharPref(name, value);
}

extern "C" void moz_pref_get_bool(const char *name, int *value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->GetBoolPref(name, value);
}

extern "C" void moz_pref_get_int(const char *name, int *value)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->GetIntPref(name, value);
}

/* clear pref */
extern "C" void moz_pref_clear(const char *name)
{
    nsCOMPtr <nsIPrefService> prefService =
	do_GetService(NS_PREFSERVICE_CONTRACTID);
    
    nsCOMPtr <nsIPrefBranch> pref;
    prefService->GetBranch("", getter_AddRefs(pref));

    if (pref)
	pref->ClearUserPref(name);
}
